@@ -12,11 +12,6 @@
 #warning Cannot find capstone-tms320c64x support
 #endif
 
-#if CS_API_MAJOR < 2
-#undef CAPSONT_HAS_TMS320C64X
-#define CAPSTONE_HAS_TMS320C64X 0
-#endif
-
 #if CAPSTONE_HAS_TMS320C64X
 
 #define INSOP(n) insn->detail->tms320c64x.operands[n]
@@ -64,7 +59,7 @@ static void opex(RzStrBuf *buf, csh handle, cs_insn *insn) {
 	pj_free(pj);
 }
 
-static int tms320c64x_analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
+static int tms320c64x_analyze_op(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
 	static csh handle = 0;
 	static int omode;
 	cs_insn *insn;
