@@ -1,5 +1,5 @@
-// SPDX-FileCopyrightText: 2021 RizinOrg <info@rizin.re>
-// SPDX-FileCopyrightText: 2021 deroad <wargio@libero.it>
+// SPDX-FileCopyrightText: 2021-2023 RizinOrg <info@rizin.re>
+// SPDX-FileCopyrightText: 2021-2023 deroad <wargio@libero.it>
 // SPDX-License-Identifier: LGPL-3.0-only
 
 #include "dex.h"
@@ -8,6 +8,8 @@
 #define DEX_INVALID_CLASS  "Lunknown_class;"
 #define DEX_INVALID_METHOD "unknown_method"
 
+#define startswith(a, b) (!strncmp(a, b, strlen(b)))
+
 typedef struct dex_access_flags_readable_t {
 	ut32 flag;
 	const char *readable;
@@ -903,18 +905,34 @@ static ut64 dex_access_flags_to_bin_flags(ut64 access_flags) {
 	return flags;
 }
 
-static char *dex_resolve_library(const char *library) {
-	if (!library || library[0] != 'L') {
+static char *demangle_java_and_free(char *mangled) {
+	if (!mangled) {
 		return NULL;
 	}
-	char *demangled = strdup(library + 1);
-	rz_str_replace_ch(demangled, '/', '.', 1);
-	if (RZ_STR_ISNOTEMPTY(demangled)) {
-		demangled[strlen(demangled) - 1] = 0;
-	}
+	char *demangled = rz_demangler_java(mangled, RZ_DEMANGLER_FLAG_ENABLE_ALL);
+	free(mangled);
 	return demangled;
 }
 
+static void set_lib_and_class_name(char *mangled, char **out_class, char **out_lib) {
+	if (!mangled) {
+		return;
+	}
+	bool is_java_lang = startswith(mangled, "Ljava/lang");
+
+	char *object = demangle_java_and_free(mangled);
+	if (!object) {
+		return;
+	}
+
+	*out_class = object;
+	if (!is_java_lang || startswith(object, "java.lang")) {
+		*out_lib = strdup(object);
+	} else {
+		*out_lib = rz_str_newf("java.lang.%s", object);
+	}
+}
+
 static RzBinSymbol *dex_method_to_symbol(RzBinDex *dex, DexEncodedMethod *encoded_method, DexMethodId *method_id, bool is_imported) {
 	RzBinSymbol *symbol = RZ_NEW0(RzBinSymbol);
 	if (!symbol) {
@@ -923,9 +941,9 @@ static RzBinSymbol *dex_method_to_symbol(RzBinDex *dex, DexEncodedMethod *encode
 
 	bool varargs = dex_is_varargs(encoded_method->access_flags);
 	symbol->name = dex_resolve_string_id(dex, method_id->name_idx);
-	symbol->classname = dex_resolve_type_id(dex, method_id->class_idx);
-	symbol->libname = dex_resolve_library(symbol->classname);
-	symbol->dname = dex_resolve_proto_id(dex, symbol->name, method_id->proto_idx, varargs);
+	char *mangled = dex_resolve_type_id(dex, method_id->class_idx);
+	set_lib_and_class_name(mangled, &symbol->classname, &symbol->libname);
+	symbol->dname = demangle_java_and_free(dex_resolve_proto_id(dex, symbol->name, method_id->proto_idx, varargs));
 	symbol->bind = dex_is_static(encoded_method->access_flags) ? RZ_BIN_BIND_GLOBAL_STR : RZ_BIN_BIND_LOCAL_STR;
 	symbol->is_imported = is_imported;
 	symbol->visibility = encoded_method->access_flags & UT32_MAX;
@@ -992,8 +1010,8 @@ static RzList /*<RzBinSymbol *>*/ *dex_resolve_methods_in_class(RzBinDex *dex, D
 	return methods;
 }
 
-static RzBinField *dex_field_to_bin_field(RzBinDex *dex, DexEncodedField *encoded_field, DexFieldId *field_id, bool is_static) {
-	RzBinField *field = RZ_NEW0(RzBinField);
+static RzBinClassField *dex_field_to_bin_field(RzBinDex *dex, DexEncodedField *encoded_field, DexFieldId *field_id, bool is_static) {
+	RzBinClassField *field = RZ_NEW0(RzBinClassField);
 	if (!field) {
 		return NULL;
 	}
@@ -1007,15 +1025,17 @@ static RzBinField *dex_field_to_bin_field(RzBinDex *dex, DexEncodedField *encode
 	field->paddr = encoded_field->offset;
 	field->visibility = encoded_field->access_flags & UT32_MAX;
 	field->visibility_str = rz_bin_dex_access_flags_readable(access_flags);
+	char *mangled = dex_resolve_type_id(dex, field_id->class_idx);
+	set_lib_and_class_name(mangled, &field->classname, &field->libname);
 	field->name = dex_resolve_string_id(dex, field_id->name_idx);
-	field->type = dex_resolve_type_id(dex, field_id->type_idx);
+	field->type = demangle_java_and_free(dex_resolve_type_id(dex, field_id->type_idx));
 	field->flags = dex_access_flags_to_bin_flags(access_flags);
 
 	return field;
 }
 
-static RzList /*<RzBinField *>*/ *dex_resolve_fields_in_class(RzBinDex *dex, DexClassDef *class_def, ut8 *inserted) {
-	RzList *fields = rz_list_newf((RzListFree)rz_bin_field_free);
+static RzList /*<RzBinClassField *>*/ *dex_resolve_fields_in_class(RzBinDex *dex, DexClassDef *class_def, ut8 *inserted) {
+	RzList *fields = rz_list_newf((RzListFree)rz_bin_class_field_free);
 	if (!fields) {
 		return NULL;
 	}
@@ -1033,9 +1053,9 @@ static RzList /*<RzBinField *>*/ *dex_resolve_fields_in_class(RzBinDex *dex, Dex
 		inserted[encoded_field->field_idx] = true;
 		field_id = (DexFieldId *)rz_pvector_at(dex->field_ids, encoded_field->field_idx);
 
-		RzBinField *field = dex_field_to_bin_field(dex, encoded_field, field_id, true);
+		RzBinClassField *field = dex_field_to_bin_field(dex, encoded_field, field_id, true);
 		if (!field || !rz_list_append(fields, field)) {
-			rz_bin_field_free(field);
+			rz_bin_class_field_free(field);
 			break;
 		}
 	}
@@ -1050,9 +1070,9 @@ static RzList /*<RzBinField *>*/ *dex_resolve_fields_in_class(RzBinDex *dex, Dex
 		inserted[encoded_field->field_idx] = true;
 		field_id = (DexFieldId *)rz_pvector_at(dex->field_ids, encoded_field->field_idx);
 
-		RzBinField *field = dex_field_to_bin_field(dex, encoded_field, field_id, false);
+		RzBinClassField *field = dex_field_to_bin_field(dex, encoded_field, field_id, false);
 		if (!field || !rz_list_append(fields, field)) {
-			rz_bin_field_free(field);
+			rz_bin_class_field_free(field);
 			break;
 		}
 	}
@@ -1066,8 +1086,8 @@ static RzBinSymbol *dex_field_to_symbol(RzBinDex *dex, DexEncodedField *encoded_
 	}
 
 	field->name = dex_resolve_string_id(dex, field_id->name_idx);
-	field->classname = dex_resolve_type_id(dex, field_id->class_idx);
-	field->libname = dex_resolve_library(field->classname);
+	char *mangled = dex_resolve_type_id(dex, field_id->class_idx);
+	set_lib_and_class_name(mangled, &field->classname, &field->libname);
 	field->bind = dex_is_static(encoded_field->access_flags) ? RZ_BIN_BIND_GLOBAL_STR : RZ_BIN_BIND_LOCAL_STR;
 	field->is_imported = false;
 	field->visibility = encoded_field->access_flags & UT32_MAX;
@@ -1122,18 +1142,6 @@ static RzList /*<RzBinSymbol *>*/ *dex_resolve_fields_in_class_as_symbols(RzBinD
 	return fields;
 }
 
-static void free_rz_bin_class(RzBinClass *bclass) {
-	if (!bclass) {
-		return;
-	}
-	rz_list_free(bclass->methods);
-	rz_list_free(bclass->fields);
-	free(bclass->name);
-	free(bclass->super);
-	free(bclass->visibility_str);
-	free(bclass);
-}
-
 /**
  * \brief Returns a RzList<RzBinClass*> containing the dex classes
  */
@@ -1156,7 +1164,7 @@ RZ_API RZ_OWN RzList /*<RzBinClass *>*/ *rz_bin_dex_classes(RZ_NONNULL RzBinDex
 		return NULL;
 	}
 
-	classes = rz_list_newf((RzListFree)free_rz_bin_class);
+	classes = rz_list_newf((RzListFree)rz_bin_class_free);
 	if (!classes) {
 		free(inserted_fields);
 		free(inserted_methods);
@@ -1170,17 +1178,16 @@ RZ_API RZ_OWN RzList /*<RzBinClass *>*/ *rz_bin_dex_classes(RZ_NONNULL RzBinDex
 			break;
 		}
 
-		bclass->name = dex_resolve_type_id(dex, class_def->class_idx);
-		bclass->super = dex_resolve_type_id(dex, class_def->superclass_idx);
+		bclass->name = demangle_java_and_free(dex_resolve_type_id(dex, class_def->class_idx));
+		bclass->super = demangle_java_and_free(dex_resolve_type_id(dex, class_def->superclass_idx));
 		bclass->visibility = class_def->access_flags;
 		bclass->visibility_str = rz_bin_dex_access_flags_readable(class_def->access_flags);
-		bclass->index = class_def->class_idx;
 		bclass->addr = class_def->offset;
 		bclass->methods = dex_resolve_methods_in_class(dex, class_def, inserted_methods);
 		bclass->fields = dex_resolve_fields_in_class(dex, class_def, inserted_fields);
 
 		if (!rz_list_append(classes, bclass)) {
-			free_rz_bin_class(bclass);
+			rz_bin_class_free(bclass);
 			break;
 		}
 	}
@@ -1236,9 +1243,9 @@ RZ_API RZ_OWN RzList /*<RzBinSection *>*/ *rz_bin_dex_sections(RZ_NONNULL RzBinD
 }
 
 /**
- * \brief Returns a RzList<RzBinField*> containing the dex fields
+ * \brief Returns a RzList<RzBinClassField*> containing the dex fields
  */
-RZ_API RZ_OWN RzList /*<RzBinField *>*/ *rz_bin_dex_fields(RZ_NONNULL RzBinDex *dex) {
+RZ_API RZ_OWN RzList /*<RzBinClassField *>*/ *rz_bin_dex_fields(RZ_NONNULL RzBinDex *dex) {
 	rz_return_val_if_fail(dex, NULL);
 
 	DexClassDef *class_def;
@@ -1251,7 +1258,7 @@ RZ_API RZ_OWN RzList /*<RzBinField *>*/ *rz_bin_dex_fields(RZ_NONNULL RzBinDex *
 		return NULL;
 	}
 
-	fields = rz_list_newf((RzListFree)rz_bin_field_free);
+	fields = rz_list_newf((RzListFree)rz_bin_class_field_free);
 	if (!fields) {
 		free(inserted);
 		return NULL;
@@ -1331,11 +1338,13 @@ RZ_API RZ_OWN RzList /*<RzBinSymbol *>*/ *rz_bin_dex_symbols(RZ_NONNULL RzBinDex
 		}
 
 		field->name = dex_resolve_string_id(dex, field_id->name_idx);
-		field->classname = dex_resolve_type_id(dex, field_id->class_idx);
-		field->libname = dex_resolve_library(field->classname);
+		char *mangled = dex_resolve_type_id(dex, field_id->class_idx);
+		set_lib_and_class_name(mangled, &field->classname, &field->libname);
 		field->bind = RZ_BIN_BIND_WEAK_STR;
 		field->type = RZ_BIN_TYPE_FIELD_STR;
 		field->is_imported = true;
+		field->vaddr = UT64_MAX;
+		field->paddr = UT64_MAX;
 
 		if (!rz_list_append(symbols, field)) {
 			rz_bin_symbol_free(field);
@@ -1356,9 +1365,9 @@ RZ_API RZ_OWN RzList /*<RzBinSymbol *>*/ *rz_bin_dex_symbols(RZ_NONNULL RzBinDex
 		}
 
 		method->name = dex_resolve_string_id(dex, method_id->name_idx);
-		method->classname = dex_resolve_type_id(dex, method_id->class_idx);
-		method->libname = dex_resolve_library(method->classname);
-		method->dname = dex_resolve_proto_id(dex, method->name, method_id->proto_idx, false);
+		char *mangled = dex_resolve_type_id(dex, method_id->class_idx);
+		set_lib_and_class_name(mangled, &method->classname, &method->libname);
+		method->dname = demangle_java_and_free(dex_resolve_proto_id(dex, method->name, method_id->proto_idx, false));
 		method->bind = RZ_BIN_BIND_WEAK_STR;
 		method->is_imported = true;
 		method->type = RZ_BIN_TYPE_METH_STR;
@@ -1437,28 +1446,12 @@ RZ_API RZ_OWN RzList /*<RzBinImport *>*/ *rz_bin_dex_imports(RZ_NONNULL RzBinDex
 			break;
 		}
 
-		char *object = dex_resolve_type_id(dex, field_id->class_idx);
-		if (RZ_STR_ISEMPTY(object)) {
-			free(object);
-			free(import);
-			break;
-		}
-		rz_str_replace_char(object, ';', 0);
-
-		char *class_name = (char *)rz_str_rchr(object, NULL, '/');
-		if (class_name) {
-			class_name[0] = 0;
-			class_name++;
-		}
-		rz_str_replace_ch(object, '/', '.', 1);
-
 		import->name = dex_resolve_string_id(dex, field_id->name_idx);
-		import->libname = class_name ? strdup(object + 1) : NULL;
-		import->classname = strdup(class_name ? class_name : object + 1);
+		char *mangled = dex_resolve_type_id(dex, field_id->class_idx);
+		set_lib_and_class_name(mangled, &import->classname, &import->libname);
 		import->bind = RZ_BIN_BIND_WEAK_STR;
 		import->type = RZ_BIN_TYPE_FIELD_STR;
 		import->ordinal = ordinal;
-		free(object);
 
 		if (!rz_list_append(imports, import)) {
 			rz_bin_import_free(import);
@@ -1485,30 +1478,14 @@ RZ_API RZ_OWN RzList /*<RzBinImport *>*/ *rz_bin_dex_imports(RZ_NONNULL RzBinDex
 			break;
 		}
 
-		char *object = dex_resolve_type_id(dex, method_id->class_idx);
-		if (RZ_STR_ISEMPTY(object)) {
-			free(object);
-			rz_bin_import_free(import);
-			break;
-		}
-		rz_str_replace_char(object, ';', 0);
-
-		char *class_name = (char *)rz_str_rchr(object, NULL, '/');
-		if (class_name) {
-			class_name[0] = 0;
-			class_name++;
-		}
-		rz_str_replace_ch(object, '/', '.', 1);
-
 		char *name = dex_resolve_string_id(dex, method_id->name_idx);
 		import->name = dex_resolve_proto_id(dex, name, method_id->proto_idx, false);
-		import->libname = class_name ? strdup(object + 1) : NULL;
-		import->classname = strdup(class_name ? class_name : object + 1);
+		char *mangled = dex_resolve_type_id(dex, method_id->class_idx);
+		set_lib_and_class_name(mangled, &import->classname, &import->libname);
 		import->bind = RZ_BIN_BIND_WEAK_STR;
 		import->type = RZ_BIN_TYPE_FUNC_STR;
 		import->ordinal = ordinal;
 		free(name);
-		free(object);
 
 		if (!rz_list_append(imports, import)) {
 			rz_bin_import_free(import);
@@ -1574,7 +1551,7 @@ RZ_API RZ_OWN RzList /*<char *>*/ *rz_bin_dex_libraries(RZ_NONNULL RzBinDex *dex
 		}
 
 		char *object = dex_resolve_type_id(dex, method_id->class_idx);
-		if (RZ_STR_ISEMPTY(object) || *object != 'L' || !strncmp(object, "Ljava/", strlen("Ljava/"))) {
+		if (RZ_STR_ISEMPTY(object) || *object != 'L' || startswith(object, "Ljava/")) {
 			free(object);
 			continue;
 		}
@@ -2028,19 +2005,19 @@ RZ_API RZ_OWN char *rz_bin_dex_version(RZ_NONNULL RzBinDex *dex) {
 	rz_return_val_if_fail(dex, NULL);
 	// https://cs.android.com/android/platform/superproject/+/master:dalvik/dx/src/com/android/dex/DexFormat.java;l=55;bpv=1;bpt=0
 	// https://developer.android.com/studio/releases/platforms
-	if (!strncmp((char *)dex->version, "009", 3)) {
+	if (startswith((char *)dex->version, "009")) {
 		return strdup("Android M3 release (Nov-Dec 2007)");
-	} else if (!strncmp((char *)dex->version, "013", 3)) {
+	} else if (startswith((char *)dex->version, "013")) {
 		return strdup("Android M5 release (Feb-Mar 2008)");
-	} else if (!strncmp((char *)dex->version, "035", 3)) {
+	} else if (startswith((char *)dex->version, "035")) {
 		return strdup("Android 3.2 (API level 13 and earlier)");
-	} else if (!strncmp((char *)dex->version, "037", 3)) {
+	} else if (startswith((char *)dex->version, "037")) {
 		return strdup("Android 7 (API level 24 and earlier)");
-	} else if (!strncmp((char *)dex->version, "038", 3)) {
+	} else if (startswith((char *)dex->version, "038")) {
 		return strdup("Android 8 (API level 26 and earlier)");
-	} else if (!strncmp((char *)dex->version, "039", 3)) {
+	} else if (startswith((char *)dex->version, "039")) {
 		return strdup("Android 9 (API level 28 and earlier)");
-	} else if (!strncmp((char *)dex->version, "040", 3)) {
+	} else if (startswith((char *)dex->version, "040")) {
 		return strdup("Android 10+ (Aug 2019)");
 	}
 	return NULL;
