@@ -3,6 +3,7 @@
 
 #include <rz_cmd.h>
 #include <rz_cons.h>
+#include <rz_core.h>
 #include <stdlib.h>
 #include "minunit.h"
 
@@ -95,7 +96,7 @@ static RzCmdStatus afl_argv_handler(RzCore *core, int argc, const char **argv) {
 }
 
 bool test_cmd_descriptor_argv(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *cd = rz_cmd_desc_argv_new(cmd, root, "afl", afl_argv_handler, &fake_help);
 	mu_assert_notnull(cd, "cmddesc created");
@@ -109,7 +110,7 @@ bool test_cmd_descriptor_argv(void) {
 }
 
 bool test_cmd_descriptor_argv_nested(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *af_cd = rz_cmd_desc_group_new(cmd, root, "af", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, root, "af2", NULL, &fake_help);
@@ -125,7 +126,7 @@ static int a_oldinput_cb(void *user, const char *input) {
 }
 
 bool test_cmd_descriptor_oldinput(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *cd = rz_cmd_desc_oldinput_new(cmd, root, "a", a_oldinput_cb, NULL);
 	mu_assert_notnull(cd, "cmddesc created");
@@ -150,7 +151,7 @@ bool test_cmd_descriptor_group(void) {
 	const RzCmdDescHelp a_exec_help = { .summary = "a exec help", .args = fake_args };
 	const RzCmdDescHelp a_group_help = { .summary = "a group help" };
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *cd = rz_cmd_desc_group_new(cmd, root, "a", a_exec_cb, &a_exec_help, &a_group_help);
 	rz_cmd_desc_argv_new(cmd, cd, "ab", ab_cb, &ab_help);
@@ -200,7 +201,7 @@ static int w_handler(void *user, const char *input) {
 }
 
 bool test_cmd_descriptor_tree(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, a_cd, "ap", ap_handler, &fake_help);
@@ -217,7 +218,7 @@ bool test_cmd_descriptor_tree(void) {
 }
 
 bool test_cmd_get_desc(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	RzCmdDesc *ap_cd = rz_cmd_desc_group_new(cmd, a_cd, "ap", ap_handler, NULL, &fake_help);
@@ -282,7 +283,7 @@ bool test_cmd_call_desc(void) {
 		.args = pd_help_args,
 	};
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
@@ -351,7 +352,7 @@ bool test_cmd_help(void) {
 		.args = fake_args,
 	};
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &p_group_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
@@ -417,7 +418,7 @@ bool test_cmd_group_help(void) {
 		.args = fake_args,
 	};
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", p_handler_argv, &p_help, &p_group_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
@@ -439,7 +440,7 @@ bool test_cmd_group_help(void) {
 bool test_cmd_oldinput_help(void) {
 	rz_cons_new();
 
-	RzCmd *cmd = rz_cmd_new(true);
+	RzCmd *cmd = rz_cmd_new(NULL, true);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &fake_help);
@@ -484,7 +485,7 @@ bool test_cmd_group_exec_help(void) {
 		.args = fake_args,
 	};
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", p_handler_argv, &p_help, &p_group_help);
 	rz_cmd_desc_argv_new(cmd, p_cd, "pd", pd_handler, &pd_help);
@@ -527,7 +528,7 @@ bool test_cmd_group_exec_help(void) {
 	mu_end;
 }
 bool test_remove_cmd(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *x_cd = rz_cmd_desc_argv_new(cmd, root, "x", NULL, &fake_help);
 	RzCmdDesc *p_cd = rz_cmd_desc_group_new(cmd, root, "p", NULL, NULL, &fake_help);
@@ -556,7 +557,7 @@ bool test_remove_cmd(void) {
 bool test_cmd_args(void) {
 	const char *x_c_choices[] = { "A", "B", "C" };
 	RzCmdDescArg x_args[] = {
-		{ .name = "c", .type = RZ_CMD_ARG_TYPE_CHOICES, .choices = x_c_choices },
+		{ .name = "c", .type = RZ_CMD_ARG_TYPE_CHOICES, .choices.choices = x_c_choices },
 		{ .name = "from", .optional = true, .type = RZ_CMD_ARG_TYPE_NUM },
 		{ .name = "to", .type = RZ_CMD_ARG_TYPE_NUM },
 		{ .name = "n", .optional = true, .type = RZ_CMD_ARG_TYPE_NUM, .default_value = "5" },
@@ -566,7 +567,7 @@ bool test_cmd_args(void) {
 	x_help.summary = "x summary";
 	x_help.args = x_args;
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *x_cd = rz_cmd_desc_argv_new(cmd, root, "x", NULL, &x_help);
 
@@ -592,7 +593,7 @@ bool test_cmd_argv_modes(void) {
 	z_help.summary = "z summary";
 	z_help.args = z_args;
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_argv_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET | RZ_OUTPUT_MODE_LONG_JSON, z_modes_handler, &z_help);
 
@@ -647,7 +648,7 @@ bool test_cmd_argv_state(void) {
 	z_help.summary = "z summary";
 	z_help.args = z_args;
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_state_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET | RZ_OUTPUT_MODE_LONG_JSON, z_state_handler, &z_help, &group_help);
 
@@ -694,7 +695,7 @@ bool test_cmd_group_argv_modes(void) {
 	RzCmdDescHelp z_group_help = { 0 };
 	z_group_help.summary = "z group summary";
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET, z_modes_handler, &z_help, &z_group_help);
 	RzCmdDesc *zd_cd = rz_cmd_desc_argv_new(cmd, z_cd, "zd", zd_handler, &fake_help);
@@ -732,7 +733,7 @@ static bool foreach_cmdname_cb(RzCmd *cmd, const RzCmdDesc *desc, void *user) {
 }
 
 bool test_foreach_cmdname(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET, z_modes_handler, &fake_help, &fake_help);
 	rz_cmd_desc_argv_new(cmd, z_cd, "zd", zd_handler, &fake_help);
@@ -773,7 +774,7 @@ bool test_foreach_cmdname(void) {
 }
 
 bool test_foreach_cmdname_begin(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET, z_modes_handler, &fake_help, &fake_help);
 	rz_cmd_desc_argv_new(cmd, z_cd, "zd", zd_handler, &fake_help);
@@ -913,7 +914,7 @@ bool test_arg_flags(void) {
 	RzCmdDescHelp x_help = { 0 };
 	x_help.summary = "x summary";
 	x_help.args = x_args;
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	rz_cmd_desc_argv_new(cmd, root, "z", z_last_handler, &z_help);
 	rz_cmd_desc_argv_new(cmd, root, "x", x_array_handler, &x_help);
@@ -952,7 +953,7 @@ bool test_get_arg(void) {
 	RzCmdDescHelp x_help = { 0 };
 	x_help.summary = "x summary";
 	x_help.args = x_args;
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_argv_new(cmd, root, "z", z_last_handler, &z_help);
 	RzCmdDesc *x_cd = rz_cmd_desc_argv_new(cmd, root, "x", x_array_handler, &x_help);
@@ -994,7 +995,7 @@ bool test_parent_details(void) {
 	RzCmdDescHelp zx_help = { 0 };
 	zx_help.summary = "x summary";
 	zx_help.args = zx_args;
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_new(cmd, root, "z", NULL, NULL, &z_group_help);
 	rz_cmd_desc_argv_new(cmd, z_cd, "zx", x_array_handler, &zx_help);
@@ -1023,7 +1024,7 @@ bool test_default_value(void) {
 	RzCmdDescHelp z_help = { 0 };
 	z_help.summary = "z summary";
 	z_help.args = z_args;
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	rz_cmd_desc_argv_new(cmd, root, "z", default_value_handler, &z_help);
 
@@ -1044,7 +1045,7 @@ bool test_sort_subcommands(void) {
 	RzCmdDescHelp z_group_help = { 0 };
 	z_group_help.summary = "z summary";
 	z_group_help.sort_subcommands = true;
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_group_new(cmd, root, "z", NULL, NULL, &z_group_help);
 	rz_cmd_desc_argv_new(cmd, z_cd, "zx", x_array_handler, &fake_help);
@@ -1255,7 +1256,7 @@ bool test_default_mode(void) {
 	x_help.summary = "x summary";
 	x_help.args = x_args;
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *z_cd = rz_cmd_desc_argv_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET | RZ_OUTPUT_MODE_LONG_JSON, z_modes_handler, &z_help);
 	RzCmdDesc *x_cd = rz_cmd_desc_argv_state_new(cmd, root, "x", RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET | RZ_OUTPUT_MODE_LONG_JSON, x_default_mode_handler, &x_help);
@@ -1306,7 +1307,7 @@ bool test_details_cb(void) {
 	z_help.args = z_args;
 	z_help.details_cb = z_details_cb;
 
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	rz_cmd_desc_argv_modes_new(cmd, root, "z", RZ_OUTPUT_MODE_STANDARD | RZ_OUTPUT_MODE_JSON | RZ_OUTPUT_MODE_QUIET | RZ_OUTPUT_MODE_LONG_JSON, z_modes_handler, &z_help);
 
@@ -1322,7 +1323,7 @@ bool test_details_cb(void) {
 }
 
 bool test_get_best_match(void) {
-	RzCmd *cmd = rz_cmd_new(false);
+	RzCmd *cmd = rz_cmd_new(NULL, false);
 	RzCmdDesc *root = rz_cmd_get_root(cmd);
 	RzCmdDesc *a_cd = rz_cmd_desc_group_new(cmd, root, "a", NULL, NULL, &fake_help);
 	RzCmdDesc *ap_cd = rz_cmd_desc_group_new(cmd, a_cd, "ap", ap_handler, NULL, &fake_help);
@@ -1340,6 +1341,131 @@ bool test_get_best_match(void) {
 	mu_end;
 }
 
+bool test_no_macros(void) {
+	RzCmd *cmd = rz_cmd_new(NULL, false);
+	RzList *l = rz_cmd_macro_list(cmd);
+	mu_assert_eq(rz_list_length(l), 0, "no macros");
+	rz_list_free(l);
+	rz_cmd_free(cmd);
+	mu_end;
+}
+
+bool test_simple_macros(void) {
+	RzCmd *cmd = rz_cmd_new(NULL, false);
+	const char *macro1_args[] = { "a", "b", NULL };
+	bool res = rz_cmd_macro_add(cmd, "macro1", macro1_args, "pd ${a} @ ${b}");
+	mu_assert_true(res, "macro1 should be added");
+	RzList *l = rz_cmd_macro_list(cmd);
+	mu_assert_eq(rz_list_length(l), 1, "no macros");
+	const RzCmdMacro *macro1 = (const RzCmdMacro *)rz_list_first(l);
+	mu_assert_streq(macro1->name, "macro1", "macro should be named macro1");
+	mu_assert_eq(macro1->nargs, 2, "macro1 should have 2 args");
+	mu_assert_streq(macro1->args[0], "a", "macro1 first arg should be a");
+	mu_assert_streq(macro1->args[1], "b", "macro1 first arg should be b");
+	mu_assert_streq(macro1->code, "pd ${a} @ ${b}", "macro1 code should be right");
+	mu_assert_ptreq(rz_cmd_macro_get(cmd, "macro1"), macro1, "_get should get the same element");
+	rz_list_free(l);
+	rz_cmd_free(cmd);
+	mu_end;
+}
+
+bool test_remove_macros(void) {
+	RzCmd *cmd = rz_cmd_new(NULL, false);
+	const char *macro1_args[] = { NULL };
+	rz_cmd_macro_add(cmd, "macro1", macro1_args, "pd ${a} @ ${b}");
+	mu_assert_notnull(rz_cmd_macro_get(cmd, "macro1"), "macro1 should be retrieved");
+	bool res = rz_cmd_macro_rm(cmd, "macro1");
+	mu_assert_true(res, "macro1 should be removed");
+	mu_assert_null(rz_cmd_macro_get(cmd, "macro1"), "macro1 should be not retrieved anymore");
+	res = rz_cmd_macro_rm(cmd, "macro1");
+	mu_assert_false(res, "macro1 should have been already removed");
+	rz_cmd_free(cmd);
+	mu_end;
+}
+
+static RzCmdStatus a_handler(RzCore *core, int argc, const char **argv) {
+	return RZ_CMD_STATUS_OK;
+}
+
+static RzCmdStatus b_handler(RzCore *core, int argc, const char **argv) {
+	if (argc != 3) {
+		return RZ_CMD_STATUS_ERROR;
+	}
+	if (!strcmp(argv[1], "20")) {
+		return !strcmp(argv[2], "10") ? RZ_CMD_STATUS_OK : RZ_CMD_STATUS_ERROR;
+	} else if (!strcmp(argv[1], "40")) {
+		return !strcmp(argv[2], "30") ? RZ_CMD_STATUS_OK : RZ_CMD_STATUS_ERROR;
+	}
+	return RZ_CMD_STATUS_ERROR;
+}
+
+bool test_call_macros(void) {
+	RzCore *core = RZ_NEW0(RzCore);
+	core->cons = rz_cons_singleton();
+	RzCmd *cmd = rz_core_cmd_new(core, false);
+	core->rcmd = cmd;
+	RzCmdDesc *root = rz_cmd_get_root(cmd);
+	rz_cmd_desc_argv_new(cmd, root, "a", a_handler, &fake_help);
+	RzCmdDescArg b_args[] = {
+		{ .name = "n1", .type = RZ_CMD_ARG_TYPE_STRING },
+		{ .name = "n2", .type = RZ_CMD_ARG_TYPE_STRING },
+		{ 0 },
+	};
+	RzCmdDescHelp b_help = {
+		.summary = "b help",
+		.args = b_args,
+	};
+	rz_cmd_desc_argv_new(cmd, root, "b", b_handler, &b_help);
+	const char *macro_args[] = { NULL };
+	const char *macro2_args[] = { "a", "b", NULL };
+	rz_cmd_macro_add(cmd, "macro1", macro_args, "a");
+	rz_cmd_macro_add(cmd, "macro2", macro2_args, "b ${b} ${a}");
+	rz_cmd_macro_add(cmd, "macro3", macro_args, "c");
+
+	const char *macro_args_val[] = { NULL };
+	const char *macro2_args_val[] = { "10", "20", NULL };
+
+	RzCmdStatus status = rz_cmd_macro_call(cmd, "macro1", macro_args_val);
+	mu_assert_eq(status, RZ_CMD_STATUS_OK, "a handler has been called correctly");
+	status = rz_cmd_macro_call(cmd, "macro2", macro2_args_val);
+	mu_assert_eq(status, RZ_CMD_STATUS_OK, "b handler has been called correctly");
+	status = rz_cmd_macro_call(cmd, "macro3", macro_args_val);
+	mu_assert_eq(status, RZ_CMD_STATUS_NONEXISTINGCMD, "c command should not exist");
+
+	rz_cmd_free(cmd);
+	mu_end;
+}
+
+bool test_call_multiple_macros(void) {
+	RzCore *core = RZ_NEW0(RzCore);
+	core->cons = rz_cons_singleton();
+	RzCmd *cmd = rz_core_cmd_new(core, false);
+	core->rcmd = cmd;
+	RzCmdDesc *root = rz_cmd_get_root(cmd);
+	RzCmdDescArg b_args[] = {
+		{ .name = "n1", .type = RZ_CMD_ARG_TYPE_STRING },
+		{ .name = "n2", .type = RZ_CMD_ARG_TYPE_STRING },
+		{ 0 },
+	};
+	RzCmdDescHelp b_help = {
+		.summary = "b help",
+		.args = b_args,
+	};
+	rz_cmd_desc_argv_new(cmd, root, "b", b_handler, &b_help);
+	const char *macro2_args[] = { "a", "b", NULL };
+	rz_cmd_macro_add(cmd, "macro2", macro2_args, "b ${b} ${a}");
+
+	const char *macro_args_val[] = { "10", "20", "30", "40", NULL };
+	const char *macro_args_val_wrong[] = { "10", "20", "30", NULL };
+
+	RzCmdStatus status = rz_cmd_macro_call_multiple(cmd, "macro2", macro_args_val);
+	mu_assert_eq(status, RZ_CMD_STATUS_OK, "macro2 has been called correctly");
+	status = rz_cmd_macro_call(cmd, "macro2", macro_args_val_wrong);
+	mu_assert_eq(status, RZ_CMD_STATUS_INVALID, "macro2 should be called with a multiple of arguments");
+	rz_cmd_free(cmd);
+	mu_end;
+}
+
 int all_tests() {
 	mu_run_test(test_parsed_args_noargs);
 	mu_run_test(test_parsed_args_onearg);
@@ -1380,6 +1506,11 @@ int all_tests() {
 	mu_run_test(test_default_mode);
 	mu_run_test(test_details_cb);
 	mu_run_test(test_get_best_match);
+	mu_run_test(test_no_macros);
+	mu_run_test(test_simple_macros);
+	mu_run_test(test_remove_macros);
+	mu_run_test(test_call_macros);
+	mu_run_test(test_call_multiple_macros);
 	return tests_passed != tests_run;
 }
 
